<?php
/**
 * Plugin Name: Payment system VTB
 * Description: Allows you to use Payment system VTB with the WooCommerce plugin.
 * Version: 1.5.11
 * Author: VTB
 * Author URI: https://www.vtb.ru/
 * Text Domain: wc-vtbpay
 * Domain Path: /languages/
 * Requires PHP: 7.4
 *
 * @package Vtbpay
 * @version 1.5.11
 */

if (!defined('ABSPATH')) exit; // Exit if accessed directly

require __DIR__ . '/vendor/autoload.php';
require __DIR__ . '/hooks.php';

use \Vtbpay\Classes\Api\VtbApi,
    \Vtbpay\Classes\Common\VtbPayLogger,
    \Vtbpay\Classes\Exception\VtbPayException;


add_action('plugins_loaded', 'init_wc_vtb_payment_gateway');

/**
 * Initialize Payment system VTB.
 */
function init_wc_vtb_payment_gateway(): void
{
    // if the WC payment gateway class is not available, do nothing
    if (!class_exists('WC_Payment_Gateway') || class_exists('WC_Vtbpay')) return;

    /**
     * Payment system VTB class.
     */
    class WC_Vtbpay extends WC_Payment_Gateway
    {
        // Payment system VTB settings
        private string $client_id;
        private string $client_secret;
        private string $merchant_authorization;
        private bool $test_mode;
        private bool $logging;
        private bool $two_stage;
        private bool $send_zip_code;
        private string $transaction_end;

        // Fiscal settings
        private bool $enable_fiscal;
        private string $email_fiscal;
        private string $payment_type_delivery_fiscal;
        private string $measure_fiscal;
        private string $tax_type_fiscal;
        private string $payment_type_fiscal;
        private string $payment_subject_fiscal;

        // Common
        private object $logger;
        private object $order;


        /**
         * Конструктор для класса плагина "Платежная система ВТБ".
         */
        function __construct()
        {
            $plugin_dir = plugin_dir_url(__FILE__);

            $this->id = 'wc_vtbpay';
            $this->icon = apply_filters('woocommerce_vtbpay_icon', $plugin_dir . 'vtbpay.png');
            $this->method_title = __('Payment system VTB', 'wc-vtbpay');
            $this->method_description = __('Plugin "VTB Payment System" for WooCommerce, which allows you to integrate online payments.', 'wc-vtbpay');
            $this->has_fields = false;

            // Load the settings
            $this->init_form_fields();
            $this->init_settings();

            // Define user set variables.
        		$this->title        = $this->get_option('title');
        		$this->description  = $this->get_option('description');
        		$this->instructions = $this->get_option('instructions');
            $this->enabled      = $this->get_option('enabled');

            // Initialize settings
            $this->init_payment_settings();

            $this->set_vtbpay_logger();

            //process settings with parent method
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
            add_action('woocommerce_api_' . $this->id . '_return', [$this, 'return_handler']);
            add_action('woocommerce_api_' . $this->id . '_webhook', [$this, 'webhook_handler']);
            // handler for checking and editing order status.
            add_action('woocommerce_api_' . $this->id . '_ajax', [$this, 'ajax_handler']);
        }


        /**
         * Инициализация настроек
         *
         * @return void
         */
        private function init_payment_settings(): void
        {
            // Settings
            $settings = [
                'client_id',
                'client_secret',
                'merchant_authorization',
                'transaction_end',
                'email_fiscal',
                'payment_type_delivery_fiscal',
                'measure_fiscal',
                'tax_type_fiscal',
                'payment_type_fiscal',
                'payment_subject_fiscal'
            ];


            foreach ($settings as $setting) {
                $this->$setting = $this->get_option($setting);
            }

            // Booleans
            $booleanSettings = [
                'test_mode',
                'logging',
                'two_stage',
                'send_zip_code',
                'enable_fiscal'
            ];

            foreach ($booleanSettings as $setting) {
                $this->$setting = $this->get_option($setting) === 'yes';
            }
        }


        /**
         * Инициализируйте поля формы настроейк платежной системы ВТБ.
         *
         * @return void
         */
        public function init_form_fields(): void
        {
            $this->form_fields = include 'form_fields.php';
        }


        /**
         * Инициализируем свойство order объектом класса WC_Order,
         * полученным с помощью идентификатора и возвращаем объект текущего класса.
         *
         * @param int $order_id Идентификатор заказа.
         *
         * @return self|Exception Объект заказа, если он существует, или Exception в противном случае.
         */
        private function set_order(int $order_id): self
        {
            if ($order = wc_get_order($order_id)) {
                $this->order = $order;
                return $this;
            }
            else {
                throw new \Exception(__('Order not found.', 'wc-vtbpay'));
            }
        }


        /**
         * Создаем новый экземпляр класса VtbApi с заданными конфигурациями платёжной системы.
         *
         * @return VtbApi The new VtbApi instance.
         */
        protected function get_vtb_api(): VtbApi
        {
            return new VtbApi(
                $this->client_id,
                $this->client_secret,
                (bool) $this->test_mode,
                $this->merchant_authorization ?: ''
            );
        }


        /**
         * Обработка оплаты заказа в WooCommerce с помощью платежной системы ВТБ.
         * должно быть совместимо с WC_Payment_Gateway::process_payment($order_id)
         *
         * @param int $order_id Идентификатор заказа.
         *
         * @return array|null Массив, содержащий результат и URL перенаправления, или null, если произошла ошибка.
         */
        public function process_payment($order_id): ?array
        {
            $this->logger->setOption('additionalCommonText', 'payment-' . rand(1111, 9999));

            try {
                $this->set_order($order_id);

                if ($this->order->get_payment_method() != $this->id) {
                    throw new \Exception(__('Payment method not "VTB Payment System".', 'wc-vtbpay'));
                }

                $pay_url = $this->get_pay_url();

                //Set order status pending payment
                $this->order->update_status('pending', __('Payment link generated:', 'wc-vtbpay') . $pay_url);

                //once the order is updated clear the cart and reduce the stock
                WC()->cart->empty_cart();

                return [
                    'result' => 'success',
                    'redirect' => $pay_url
                ];

            } catch (\Exception | VtbPayException $e) {
                // Handle exception and log error
                $context = [
                    'file_exception' => $e->getFile(),
                    'line_exception' => $e->getLine(),
                ];
                if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

                $this->logger->error(sprintf(
    								__FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
    								$e->getMessage(),
    								$order_id ?: ''
    						), $context);

                wc_add_notice($e->getMessage(), 'error');
                return null;
            }
        }


        /**
         * Получает URL-адрес платежа для перенаправления.
         *
         * @return string URL-адрес платежа.
         */
        private function get_pay_url(): string
        {
            $order_id = $this->order->get_id();
            $email = $this->order->get_billing_email() ?: $this->email_fiscal;
            $total = $this->order->get_total();
            $return_url = home_url(sprintf('/wc-api/%s_return', $this->id)); // Get return URL

            // Указываем индекс почты как добавочный текст к названию заказа
            $appended_order_name = '';
            if ($this->send_zip_code) {
                $appended_order_name = '. ' . __('Postal code', 'wc-vtbpay') . ': ';
                $appended_order_name .= $this->order->get_shipping_postcode() ?: $this->order->get_billing_postcode() ?: '';
            }

            $items = [];
            if (!$this->two_stage && $this->enable_fiscal) $items = $this->get_items();

    				// Logging input
    				$this->logger->debug(
                __FUNCTION__ . ' > getOrderLink - INPUT: ', [
    						'arguments' => [
    								'order_id' => $order_id,
    								'email' => $email,
    								'time' => time(),
    								'total' => $total,
    								'return_url' => $return_url,
                    'two_stage' => $this->two_stage,
                    'items' => $items,
                    'appended_order_name' => $appended_order_name

    						]
    				]);

            $response = $this->get_vtb_api()->getOrderLink(
                $order_id,
                $email,
                time(),
                $total,
                $return_url,
                $this->two_stage,
                $items,
                $appended_order_name
            );

    				// Logging output
    				$this->logger->debug(
                __FUNCTION__ . ' > getOrderLink - OUTPUT: ', [
    						'response' => $response
    				]);

            return $response['object']['payUrl'] ?? '';
        }


        /**
         * Отображение описания платежной системы ВТБ при оформлении заказа.
         *
         * @return void
         */
        public function payment_fields(): void
        {
            if (!empty($this->description)) {
                echo wpautop(wptexturize($this->description));
            }
        }


        /**
         * Обработчик return, вызываемый при переходе на страницу return_url, после попытки оплаты.
         *
         * @return void
         */
        public function return_handler(): void
        {
            $this->logger->setOption('additionalCommonText', 'return-' . rand(1111, 9999));
            $order_id = $_REQUEST['orderId'] ?? null;

            try {
                if (empty($order_id)) {
                    throw new \Exception(__('Order ID is empty.', 'wc-vtbpay'));
                }

                // Logging $_REQUEST
                $this->logger->debug(
                    __FUNCTION__ . ' > return: ', [
                    'request_data' => $_REQUEST
                ]);

                $this->set_order($order_id);

                $payment_status = $this->get_payment_status();

                $this->change_payment_status($payment_status);

                if (in_array($payment_status, ['PAID', 'PENDING', 'RECONCLIED'])) {
                    $redirect_url = $this->get_return_url($this->order);
                }
                else {
                    $redirect_url = str_replace('&amp;', '&', $this->order->get_cancel_order_url());
                }

                wp_safe_redirect(esc_url($redirect_url));

            } catch (\Exception | VtbPayException $e) {
                // Handle exception and log error
                $context = [
                    'file_exception' => $e->getFile(),
                    'line_exception' => $e->getLine(),
                ];
                if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

                $this->logger->error(sprintf(
                    __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
                    $e->getMessage(),
                    $order_id ?: ''
                ), $context);

                wp_die($e->getMessage());
            }
        }


        /**
         * Обработчик return, вызываемый при переходе на страницу return_url, после попытки оплаты.
         *
         * @return void
         */
        public function webhook_handler(): void
        {
            $this->logger->setOption('additionalCommonText', 'webhook-' . rand(1111, 9999));
            $php_input = json_decode(file_get_contents('php://input'), true) ?: null;

            $order_id = $php_input['object']['orderId'] ?? '';

            try {
                if (empty($order_id)) {
                    throw new \Exception(__('Order ID is empty.', 'wc-vtbpay'));
                }

                // Logging php input
                $this->logger->debug(
                    __FUNCTION__ . ' > callback: ', [
                    'php_input' => $php_input
                ]);

                $this->set_order($order_id);

                if ($this->order->get_status() === $php_input['object']['status']['value']) die('OK');

                $payment_status = $this->get_payment_status();

                $this->change_payment_status($payment_status);

                die('OK');

            } catch (\Exception | VtbPayException $e) {
                // Handle exception and log error
                $context = [
                    'file_exception' => $e->getFile(),
                    'line_exception' => $e->getLine(),
                ];
                if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

                $this->logger->error(sprintf(
                    __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
                    $e->getMessage(),
                    $order_id ?: ''
                ), $context);

                wp_die($e->getMessage());
            }
        }


        /**
         * Устанавливаем статус заказа
         *
         * @param string $payment_status Статус платежа
         *
         * @return bool
         */
        private function change_payment_status(string $payment_status): bool
        {
            $available_statuses = [
                'PAID' => fn() => $this->actions_for_paid_order(),
                'PENDING' => fn() => $this->actions_for_hold_order(),
                'REFUNDED' => fn() => $this->actions_for_refunded_order(),
                'PARTIALLY_REFUNDED' => fn() => $this->actions_for_partially_refunded_order()
            ];

            $order_status_fnc = $available_statuses[$payment_status] ?? false;

            if (!empty($order_status_fnc)) $order_status_fnc();
            else $this->actions_for_unpaid_order(); // Payment not completed

            return !empty($order_status_fnc);
        }


        /**
         * Проверяем статус платежа, и если он PAID, то устанавливаем заказ в CMS как оплаченный.
         * Если статус платежа REFUNDED, то отправляем в Первый ОФД скорректированный чек,
         * в котором указано, что он отменён и в CMS устанавливаем заказ в как отменённый.
         *
         * @return void
         */
        public function ajax_handler(): void
        {
            $this->logger->setOption('additionalCommonText', 'ajax-' . rand(1111, 9999));
            // Получает данные из AJAX-запроса через $_POST
            $order_id = $_POST['order_id'] ?? '';
            $action = $_POST['action'] ?? '';

            try {
                if (!wp_verify_nonce($_POST['nonce'] ?? '', 'vtbpay-ajax-nonce')) {
                    throw new \Exception(__('Failed ajax validation.', 'wc-vtbpay'));
                }

                if (empty($order_id)) {
                    throw new \Exception(__('Order ID is empty.', 'wc-vtbpay'));
                }

                if (empty($action)) {
                    throw new \Exception(__('Required action not specified.', 'wc-vtbpay'));
                }

                // Logging php input
                $this->logger->debug(
                    __FUNCTION__ . ' > ajax: ', [
                    'post' => $_POST
                ]);

                $this->set_order($order_id);

                $payment_status = $this->get_payment_status();

                if ($action == 'refund') {
                    $this->make_chargeback();

                    $message = __('Payment refunded.', 'wc-vtbpay');
                }
                elseif ($action == 'check_status') {
                    $this->change_payment_status($payment_status);

                    $message = __('Status updated.', 'wc-vtbpay');
                }

                // Отправьте JSON-ответ и завершите выполнение скрипта
                wp_send_json([
                    'success' => true,
                    'message' => $message
                ]);

            } catch (\Exception | VtbPayException $e) {
                // Handle exception and log error
                $context = [
                    'file_exception' => $e->getFile(),
                    'line_exception' => $e->getLine(),
                ];
                if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

                $this->logger->error(sprintf(
                    __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
                    $e->getMessage(),
                    $order_id ?: ''
                ), $context);

                wp_send_json([
                    'success' => false,
                    'message' => $e->getMessage()
                ]);
            }
        }


        /**
         * Отправляем запрос на возврат в ВТБ, возвращвем результат запроса и логируем входящие и выходящие данные.
         * Если запрос прошёл успешно, то и в CMS отображаем информацию о возврате.
         *
         * @return array
         */
        private function make_chargeback(): array
        {
            $order_id = $this->order->get_id();

            $amount = $_POST['refund_amount'] ?? 0;

            if (empty($amount)) throw new \Exception(__('Refund amount not specified.', 'wc-vtbpay'));

            $amount = self::parse_amount($amount);

            $email = $this->order->get_billing_email() ?: $this->email_fiscal;

            $items = [];
            if (!$this->two_stage && $this->enable_fiscal) $items = $this->preparing_items_for_refunds(
                $this->get_items()
            );

            // Logging input
            $this->logger->debug(
                __FUNCTION__ . ' > setRefunds - INPUT: ', [
                'arguments' => [
                    'order_id' => $order_id,
                    'amount' => $amount,
                    'email' => $email,
                    'items' => $items
                ]
            ]);

            $response = $this->get_vtb_api()->setRefunds(
                $order_id,
                $amount,
                $email,
                $items
            );

            // Logging output
            $this->logger->debug(
                __FUNCTION__ . ' > setRefunds - OUTPUT: ', [
                'response' => $response
            ]);

            return $response;
        }


        /**
         * Подготавливает элементы для возврата.
         *
         * @param array $items Элементы для подготовки.
         * @return array Массив подготовленных элементов.
         * @throws \Exception Если отсутствуют необходимые данные для возврата.
         */
         private function preparing_items_for_refunds(array $items): array
         {
             $refund_order_item_qty = $_POST['refund_order_item_qty'] ?? [];
             $refund_line_total = $_POST['refund_line_total'] ?? [];

             if (empty($refund_order_item_qty) || empty($refund_line_total)) {
                 throw new \Exception(__('The required data for a refund was not transmitted.', 'wc-vtbpay'));
             }

             $prepared_items = [];

             foreach ($items as $item) {
                 $item_id = explode('-', $item['code'])[0] ?? '';
                 $quantity = $refund_order_item_qty[$item_id] ?? 0;
                 $amount = $refund_line_total[$item_id] ?? 0;

                 if (empty($quantity) && empty($amount)) continue;

                 if (!empty($amount)) $item['amount'] = self::parse_amount($amount);
                 else $item['amount'] = (int) $quantity * $item['price'];

                 if (!empty($quantity)) $item['quantity'] = (int) $quantity;

                 $item['price'] = $item['amount'] / $item['quantity'];

                 // Добавляем элемент в новый массив
                 $prepared_items[] = $item;
             }

             return $prepared_items;
         }


        /**
         * Преобразует строку в число с плавающей запятой.
         *
         * @param mixed $amount Сумма для преобразования.
         * @return float Преобразованное значение суммы.
         */
        private static function parse_amount($amount): float
        {
            if (is_string($amount)) {
                $amount = str_replace([' ', ','], ['', '.'], $amount);
            }

            return floatval($amount);
        }


        /**
         * Получает массив товаров заказа.
         *
         * @return array Массив товаров.
         */
        private function get_items(): array
        {
            // Настройки по умолчанию для фискальных параметров
            $fiscal_settings = [
                'measure_fiscal' => 0,
                'tax_type_fiscal' => 'none',
                'payment_type_fiscal' => 'full_prepayment',
                'payment_subject_fiscal' => 1
            ];
            $items = [];
            $key = 1;

            foreach ($this->order->get_items() as $item_id => $item) {

                $product_id = $item->get_product_id();

                $product = $item->get_product();

                $amount = floatval($item->get_total() + $item->get_total_tax()); // Общая сумма по товару с учётом скидок и налогов
                $price = floatval($amount / $item->get_quantity());  // Цена товара с учётом скидок и налогов

                $product_fiscal = [];
                // Получение фискальных параметров для каждого продукта
                foreach ($fiscal_settings as $name => $default) {
                    $product_fiscal[$name] = str_replace('_' . $name, '', get_post_meta($product_id, 'vtbpay_' . $name, true)) ?: '-';
                    if ($product_fiscal[$name] === '-') $product_fiscal[$name] = $this->$name ?: $default;
                }

                $items[] = [
                    'positionId' => $key,
                    'name' => $item->get_name(),
                    'code' => (string) $item_id . '-' . $product_id,
                    'price' => $price,
                    'measure' => (int) $product_fiscal['measure_fiscal'],
                    'quantity' => (int) $item->get_quantity(),
                    'taxParams' => [
                        'taxType' => $product_fiscal['tax_type_fiscal']
                    ],
                    'paymentType' => $product_fiscal['payment_type_fiscal'],
                    'paymentSubject' => (int) $product_fiscal['payment_subject_fiscal'],
                    'amount' => $amount
                ];

                $key++;
            }


            foreach ($this->order->get_items('shipping') as $item_id => $item) {
                $amount = $item->get_total() + $item->get_total_tax();

                if ($amount <= 0) continue;

                $items[] = [
                    'positionId' => $key,
                    'name' => $item->get_name(),
                    'code' => (string) $item_id . '-' . $item->get_method_id(),
                    'price' => $amount,
                    'measure' => 0,
                    'quantity' => 1,
                    'taxParams' => [
                        'taxType' => $this->tax_type_fiscal ?: 'none'
                    ],
                    'paymentType' => $this->payment_type_delivery_fiscal ?: 'full_prepayment',
                    'paymentSubject' => 4,
                    'amount' => $amount
                ];

                $key++;
            }

            return $items;
        }


        /**
         * Обработка действий для оплаченного заказа.
         *
         * @return void
         */
        private function actions_for_paid_order(): void
        {
            $completed_status = str_replace('wc-', '', $this->transaction_end);
            if ($this->order->get_status() !== $completed_status) {
                $this->order->update_status(
                    $completed_status,
                    __('Payment completed successfully.', 'wc-vtbpay')
                );
                $this->order->payment_complete();
                wc_reduce_stock_levels($this->order->get_id());
            }
        }


        /**
         * Обработка действий для удержанного заказа.
         *
         * @return void
         */
        private function actions_for_hold_order(): void
        {
            if ($this->order->get_status() !== 'on-hold') $this->order->update_status(
                'on-hold',
                __('Payment received but not confirmed.', 'wc-vtbpay')
            );
        }


        /**
         * Обработка действий в случае неоплаченного заказа.
         *
         * @return void
         */
        private function actions_for_unpaid_order(): void
        {
            // Logging an unpaid order
            $this->logger->debug(sprintf(
                __FUNCTION__ . '. ' . __('Payment not paid.', 'wc-vtbpay') . '; Order ID: %s',
                $this->order->get_id()
            ));

            if ($this->order->get_status() !== 'failed') $this->order->update_status(
                'failed',
                __('Payment not paid.', 'wc-vtbpay')
            );
            wc_add_notice(__('Payment not paid. Your order has been canceled.', 'wc-vtbpay'), 'error');
        }


        /**
         * Обработка действий в случае возвращённого заказа.
         *
         * @return void
         */
        private function actions_for_refunded_order(): void
        {
            // Logging an unpaid order
            $this->logger->debug(sprintf(
                __FUNCTION__ . '. ' . __('Refund of payment.', 'wc-vtbpay') . '; Order ID: %s',
                $this->order->get_id()
            ));

            if ($this->order->get_status() !== 'refunded') {
              $this->order->update_status(
                  'refunded',
                  __('Refund of payment.', 'wc-vtbpay')
              );
            }
            wc_add_notice(
                __('The payment was refunded.', 'wc-vtbpay'),
                'notice'
            );
        }


        /**
         * Обработка действий в случае частично возвращённого заказа.
         *
         * @return void
         */
        private function actions_for_partially_refunded_order(): void
        {
            // Logging an unpaid order
            $this->logger->debug(sprintf(
                __FUNCTION__ . '. ' . __('Partial refund of payment.', 'wc-vtbpay') . '; Order ID: %s',
                $this->order->get_id()
            ));

            $completed_status = str_replace('wc-', '', $this->transaction_end);
            if ($this->order->get_status() !== $completed_status) {
              $this->order->update_status(
                  $completed_status,
                  __('Partial refund of payment.', 'wc-vtbpay')
              );
            }
            wc_add_notice(
                __('The payment was partial refunded.', 'wc-vtbpay'),
                'notice'
            );
        }


        /**
         * Получение статуса заказа WooCommerce из VTB API.
         *
         * @return string Значение статуса заказа, если оно существует, или пустая строка в противном случае.
         */
        private function get_payment_status(): string
        {
            $order_id = $this->order->get_id();

            // Logging input
            $this->logger->debug(
                __FUNCTION__ . ' > getOrderInfo - INPUT: ', [
                'arguments' => [
                    'order_id' => $order_id
                ]
            ]);

            $response = $this->get_vtb_api()->getOrderInfo($order_id);

            // Logging output
    				$this->logger->debug(
                __FUNCTION__ . ' > getOrderInfo - OUTPUT: ', [
    						'response' => $response
    				]);

            return $response['object']['status']['value'] ?? '';
        }


        /**
         * Инициализация и настройка объекта класса VtbPayLogger.
         *
         * Эта функция инициализирует и настраивает логгер, используемый плагином VtbPay для ведения журнала.
         *
         * @return void
         */
    		private function set_vtbpay_logger(): void
        {
            if (version_compare(WOOCOMMERCE_VERSION, '2.1', '<')) {
                global $woocommerce;
                $wc_log = $woocommerce->logger();
            }
            else {
                $wc_log = new WC_Logger();
            }

            // Настройка в админке (логироать или нет)
            $logging = $this->logging;

            $this->logger = VtbPayLogger::getInstance()
                            ->setOption('showCurrentDate', false)
                            ->setOption('showLogLevel', false)
                            ->setCustomRecording(function($message) use ($wc_log){
                                $wc_log->error($message);
                            }, VtbPayLogger::LOG_LEVEL_ERROR)
                            ->setCustomRecording(function($message) use ($wc_log, $logging){
                                if ($logging) $wc_log->debug($message);
                            }, VtbPayLogger::LOG_LEVEL_DEBUG);
        }
    }
}
